@@ -48,6 +48,7 @@ coff_init(void)
       state.obj.object->filename = strdup(state.objfilename);
       state.obj.object->processor = state.processor;
       state.obj.object->class = state.device.class;
+      state.obj.object->isnew = state.obj.newcoff;
       state.obj.enabled = true;
     }
   }
@@ -144,29 +145,18 @@ _copy_config(void)
     
     assert(config_section != NULL);
 
-    if(_16bit_core) {
-      config_section->size = 0;    
-      start = config_section->address >> 1;
-      stop = CONFIG7H >> 1;
-      for (i = start; i <= stop; i++) {
-        word = i_memory_get(state.c_memory, i);
-        if (word & MEM_USED_MASK) {
-          i_memory_put(config_section->data, i, word);
-        } else {
-          /* fill undefined configuration registers with 0xff */
-          i_memory_put(config_section->data, i, 0xffff | MEM_USED_MASK);
-        }
-        config_section->size += 2;    
+    start = state.processor_info->config_addrs[0] >> _16bit_core;
+    stop = state.processor_info->config_addrs[1] >> _16bit_core;
+    config_section->size = (stop - start + 1) * 2;
+    for (i = start; i <= stop; i++) {
+      word = i_memory_get(state.c_memory, i);
+      if (word & MEM_USED_MASK) {
+        i_memory_put(config_section->data, i, word);
+      } else {
+        /* fill undefined configuration registers with 0xff */
+        i_memory_put(config_section->data, i, 0xffff | MEM_USED_MASK);
       }
-      
-    } else {
-      /* FIXME: Some 16xx devices have two config words (16f88). */
-      word = i_memory_get(state.c_memory, state.device.config_address);
-      assert(word & MEM_USED_MASK);
-      i_memory_put(config_section->data, state.device.config_address, word);
-      config_section->size = 2;    
-    }  
-
+    }
     _update_section_symbol(config_section);
   }
 
@@ -370,6 +360,7 @@ coff_linenum(int emitted)
   gp_linenum_type *new = NULL;
   int i;
   int origin;
+  static gp_boolean show_bad_debug = true;
 
   if ((!state.obj.enabled) || (state.obj.section == NULL))
     return;
@@ -382,11 +373,18 @@ coff_linenum(int emitted)
     origin = (state.lst.line.was_org - state.obj.section->address) << _16bit_core;
   }
   
+  if (state.debug_info && (state.obj.debug_file == NULL)) {
+    if (show_bad_debug) {
+      gperror(GPE_UNKNOWN, ".file directive required to generate debug info");
+      show_bad_debug = false;
+    }
+    return;
+  }
+
   for (i = 0; i < emitted; i++) {
      
     new = gp_coffgen_addlinenum(state.obj.section);
     if (state.debug_info) {
-      assert(state.obj.debug_file != NULL);
       new->symbol = state.obj.debug_file;
       new->line_number = state.obj.debug_line;
     } else {
