@@ -74,6 +74,11 @@ enum gpasm_modes {
   relocatable
 };
 
+struct range_pair {
+  long start, end;
+  struct range_pair *next;
+};
+
 extern struct gpasm_state {
   enum gpasm_modes mode;
   gp_boolean extended_pic16e;
@@ -103,6 +108,7 @@ extern struct gpasm_state {
   gp_boolean found_idlocs;	/* idlocs directive in source code */
   unsigned int maxram;		/* Highest legal memory location */
   long maxrom;			/* Highest legal program memory location */
+  struct range_pair *badrom;
   enum outfile
     codfile,			/* Symbol output file control */
     depfile,			/* Dependency output file control */
@@ -199,6 +205,7 @@ extern struct gpasm_state {
     int flags;			/*   Current section flags */
     gp_symbol_type *debug_file; /*   Debug information for high level langs */
     unsigned int debug_line;    
+    int newcoff;
   } obj;
   struct source_context *src;	/* Top of the stack of source files */
   struct file_context *files;   /* Top of the stack of all files */
@@ -216,12 +223,15 @@ extern struct gpasm_state {
   } next_buffer;
 } state;
 
+enum globalLife { TEMPORARY, PERMANENT };
+
 struct variable {
   int value;
   int coff_num;
   int coff_section_num;
   enum gpasmValTypes type;
   enum gpasmValTypes previous_type; /* can change from static to global */
+  enum globalLife lifetime;
 };
 
 /************************************************************************/
@@ -289,8 +299,6 @@ struct source_context {
   struct source_context *prev;
 };
 
-enum globalLife { TEMPORARY, PERMANENT };
-
 void yyerror(char *s);
 int stringtolong(char *string, int radix);
 int gpasm_magic(char *);
@@ -302,6 +310,7 @@ void set_global(char *name,
 		gpasmVal value,
 		enum globalLife lifetime,
 		enum gpasmValTypes type);
+void purge_temp_symbols(struct symbol_table *table);
 void select_errorlevel(int level);
 void select_expand(char *expand);
 void select_hexformat(char *format_name);
