@@ -147,6 +147,8 @@ char *gp_geterror(unsigned int code)
     return "Each object file section must be contiguous.";
   case GPE_MUST_BE_LABEL:
     return "Operand must be an address label.";
+  case GPE_ORG_ODD:
+    return "ORG at odd address.";
   case GPE_FILL_ODD:
     return "Cannot use FILL Directive with odd number of bytes.";
   case GPE_CONTIG_CONFIG:
@@ -155,6 +157,10 @@ char *gp_geterror(unsigned int code)
     return "__IDLOC directives must be contiguous.";
   case GPE_MISSING_BRACKET:
     return "Square brackets required around offset operand.";
+  case GPE_CONFIG_UNKNOWN:
+    return "An error with the CONFIG directive occured.";
+  case GPE_CONFIG_usCONFIG:
+    return "You cannot mix CONFIG and __CONFIG directives.";
   case GPE_UNKNOWN:
   default:
     return "UNKNOWN";
@@ -235,6 +241,8 @@ char *gp_getwarning(unsigned int code)
     return "Expected dec, oct, hex. Will use hex.";
   case GPW_INVALID_RAM:
     return "Invalid RAM location specified.";
+  case GPW_EXCEED_ROM:
+    return "Address exceeds maximum range for this processor.";
   case GPW_DISABLE_ERROR:
     return "Error messages cannot be disabled.";
   case GPW_REDEFINING_PROC:
@@ -323,7 +331,7 @@ char *gp_getmessage(unsigned int code)
   case GPM_EXTPAGE:
     return "Page or Bank selection not needed for this device.";
   case GPM_CBLOCK:
-    return "CBLOCK will begin at address 0.";
+    return "CBLOCK constants will start with a value of 0.";
   case GPM_W_MODIFIED:
     return "W Register modified.";
   case GPM_SPECIAL_MNEMONIC:
