@@ -204,6 +204,8 @@ void yyerror(char *message)
 
 int yylex(void);
 extern int _16bit_core;
+extern gp_boolean _16packed_byte_acc;
+static gp_boolean _16packed_offset_labels;
 
 /************************************************************************/
 
@@ -405,14 +407,14 @@ void next_line(int value)
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 225 "parse.y"
+#line 227 "parse.y"
 {
   gpasmVal i;
   char *s;
   struct pnode *p;
 }
 /* Line 187 of yacc.c.  */
-#line 416 "parse.c"
+#line 418 "parse.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -425,7 +427,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 429 "parse.c"
+#line 431 "parse.c"
 
 #ifdef short
 # undef short
@@ -757,19 +759,19 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   323,   323,   327,   326,   332,   339,   351,   360,   369,
-     378,   443,   455,   455,   458,   467,   467,   474,   479,   479,
-     486,   486,   493,   499,   504,   509,   514,   519,   535,   552,
-     551,   570,   569,   587,   593,   595,   602,   604,   611,   620,
-     631,   633,   637,   644,   652,   652,   652,   653,   653,   653,
-     653,   654,   654,   654,   657,   662,   667,   672,   677,   684,
-     686,   694,   697,   699,   707,   710,   712,   720,   723,   725,
-     733,   733,   733,   736,   738,   746,   746,   746,   746,   746,
-     746,   749,   751,   759,   759,   762,   764,   772,   772,   775,
-     777,   785,   785,   785,   788,   790,   797,   797,   797,   797,
-     797,   797,   797,   800,   805,   810,   815,   820,   825,   830,
-     835,   840,   845,   852,   857,   862,   870,   875,   883,   893,
-     898,   906,   905,   917
+       0,   325,   325,   329,   328,   334,   341,   353,   362,   371,
+     380,   451,   463,   463,   466,   475,   475,   482,   487,   487,
+     494,   494,   501,   507,   512,   517,   522,   527,   543,   560,
+     559,   578,   577,   595,   601,   603,   610,   612,   619,   628,
+     639,   641,   645,   652,   660,   660,   660,   661,   661,   661,
+     661,   662,   662,   662,   665,   670,   675,   680,   685,   692,
+     694,   702,   705,   707,   715,   718,   720,   728,   731,   733,
+     741,   741,   741,   744,   746,   754,   754,   754,   754,   754,
+     754,   757,   759,   767,   767,   770,   772,   780,   780,   783,
+     785,   793,   793,   793,   796,   798,   805,   805,   805,   805,
+     805,   805,   805,   808,   813,   818,   823,   828,   833,   838,
+     843,   848,   853,   860,   865,   870,   878,   893,   902,   913,
+     918,   926,   925,   937
 };
 #endif
 
@@ -1857,7 +1859,7 @@ yyreduce:
   switch (yyn)
     {
         case 3:
-#line 327 "parse.y"
+#line 329 "parse.y"
     { 
           state.lst.line.was_org = state.org; 
           state.lst.line.linetype = none; 
@@ -1866,14 +1868,14 @@ yyreduce:
     break;
 
   case 5:
-#line 333 "parse.y"
+#line 335 "parse.y"
     { 
 	  next_line(0);
 	}
     break;
 
   case 6:
-#line 340 "parse.y"
+#line 342 "parse.y"
     {
 	  struct pnode *parms;
 	  int exp_result;
@@ -1887,7 +1889,7 @@ yyreduce:
     break;
 
   case 7:
-#line 352 "parse.y"
+#line 354 "parse.y"
     {
 	  struct pnode *parms;
 
@@ -1898,7 +1900,7 @@ yyreduce:
     break;
 
   case 8:
-#line 361 "parse.y"
+#line 363 "parse.y"
     {
 	  struct pnode *parms;
 
@@ -1909,7 +1911,7 @@ yyreduce:
     break;
 
   case 9:
-#line 370 "parse.y"
+#line 372 "parse.y"
     {
           struct pnode *parms;
 
@@ -1920,10 +1922,16 @@ yyreduce:
     break;
 
   case 10:
-#line 379 "parse.y"
+#line 381 "parse.y"
     {
-	  if (asm_enabled() && (state.lst.line.linetype == none))
-	    state.lst.line.linetype = insn;
+	  if (asm_enabled() && (state.lst.line.linetype == none)) {
+	    if ((state.mode == relocatable) &&
+                (SECTION_FLAGS & (STYP_BSS | STYP_DATA)))
+          /* alias to next definition */
+          state.lst.line.linetype = res;
+        else
+	      state.lst.line.linetype = insn;
+	  }
 	  
 	  if (asm_enabled()) {
 	    if (state.mac_head) {
@@ -1969,7 +1977,7 @@ yyreduce:
 		set_global((yyvsp[(1) - (2)].s), (yyvsp[(2) - (2)].i), PERMANENT, gvt_constant);
 		break;
 	      case insn:
-		set_global((yyvsp[(1) - (2)].s), (yyvsp[(2) - (2)].i) << _16bit_core, PERMANENT, gvt_address);
+		set_global((yyvsp[(1) - (2)].s), ((yyvsp[(2) - (2)].i) << _16bit_core) - _16packed_offset_labels, PERMANENT, gvt_address);
 		break;
 	      case res:
 		set_global((yyvsp[(1) - (2)].s), (yyvsp[(2) - (2)].i), PERMANENT, gvt_static);
@@ -1987,7 +1995,7 @@ yyreduce:
     break;
 
   case 11:
-#line 444 "parse.y"
+#line 452 "parse.y"
     {
 	  if (state.mac_head) {
 	    /* This is a macro definition, but the label was missing */
@@ -2000,7 +2008,7 @@ yyreduce:
     break;
 
   case 14:
-#line 459 "parse.y"
+#line 467 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    (yyval.i) = state.org;
@@ -2011,12 +2019,12 @@ yyreduce:
     break;
 
   case 15:
-#line 467 "parse.y"
+#line 475 "parse.y"
     {  force_ident = 1; }
     break;
 
   case 16:
-#line 469 "parse.y"
+#line 477 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (4)].s), mk_list(mk_symbol((yyvsp[(3) - (4)].s)), NULL));
 	  force_ident = 0;
@@ -2024,19 +2032,19 @@ yyreduce:
     break;
 
   case 17:
-#line 475 "parse.y"
+#line 483 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (2)].s), NULL);
 	}
     break;
 
   case 18:
-#line 479 "parse.y"
+#line 487 "parse.y"
     {  force_decimal = 1; }
     break;
 
   case 19:
-#line 481 "parse.y"
+#line 489 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (4)].s), (yyvsp[(3) - (4)].p));
 	  force_decimal = 0;
@@ -2044,12 +2052,12 @@ yyreduce:
     break;
 
   case 20:
-#line 486 "parse.y"
+#line 494 "parse.y"
     {  force_decimal = 1; }
     break;
 
   case 21:
-#line 488 "parse.y"
+#line 496 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (4)].s), (yyvsp[(3) - (4)].p));
 	  force_decimal = 0;
@@ -2057,7 +2065,7 @@ yyreduce:
     break;
 
   case 22:
-#line 494 "parse.y"
+#line 502 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (4)].s), mk_list(mk_string((yyvsp[(2) - (4)].s)),
 	      mk_list(mk_string((yyvsp[(3) - (4)].s)), NULL)));
@@ -2065,35 +2073,35 @@ yyreduce:
     break;
 
   case 23:
-#line 500 "parse.y"
+#line 508 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (3)].s), mk_list(mk_string((yyvsp[(2) - (3)].s)), NULL));
 	}
     break;
 
   case 24:
-#line 505 "parse.y"
+#line 513 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (2)].s), NULL);
 	}
     break;
 
   case 25:
-#line 510 "parse.y"
+#line 518 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (2)].s), NULL);
 	}
     break;
 
   case 26:
-#line 515 "parse.y"
+#line 523 "parse.y"
     {
 	  (yyval.i) = do_or_append_insn((yyvsp[(1) - (3)].s), (yyvsp[(2) - (3)].p));
 	}
     break;
 
   case 27:
-#line 520 "parse.y"
+#line 528 "parse.y"
     {
 	  int number;
 	  int i;
@@ -2111,7 +2119,7 @@ yyreduce:
     break;
 
   case 28:
-#line 536 "parse.y"
+#line 544 "parse.y"
     {
 	  int number;
 	  int i;
@@ -2129,7 +2137,7 @@ yyreduce:
     break;
 
   case 29:
-#line 552 "parse.y"
+#line 560 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    begin_cblock((yyvsp[(2) - (3)].p));
@@ -2141,7 +2149,7 @@ yyreduce:
     break;
 
   case 30:
-#line 562 "parse.y"
+#line 570 "parse.y"
     {
 	  if (state.mac_prev) {
 	    macro_append();
@@ -2151,7 +2159,7 @@ yyreduce:
     break;
 
   case 31:
-#line 570 "parse.y"
+#line 578 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    continue_cblock();
@@ -2163,7 +2171,7 @@ yyreduce:
     break;
 
   case 32:
-#line 580 "parse.y"
+#line 588 "parse.y"
     {
 	  if (state.mac_prev) {
 	    macro_append();
@@ -2173,21 +2181,21 @@ yyreduce:
     break;
 
   case 33:
-#line 588 "parse.y"
+#line 596 "parse.y"
     {
 	  (yyval.i) = 0;
   	}
     break;
 
   case 35:
-#line 596 "parse.y"
+#line 604 "parse.y"
     {
 	  next_line(0);
 	}
     break;
 
   case 37:
-#line 605 "parse.y"
+#line 613 "parse.y"
     {
 	  if (state.mac_prev) {
 	    macro_append();
@@ -2196,7 +2204,7 @@ yyreduce:
     break;
 
   case 38:
-#line 612 "parse.y"
+#line 620 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    cblock_expr(mk_symbol((yyvsp[(1) - (2)].s)));
@@ -2207,7 +2215,7 @@ yyreduce:
     break;
 
   case 39:
-#line 621 "parse.y"
+#line 629 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    cblock_expr_incr(mk_symbol((yyvsp[(1) - (3)].s)), (yyvsp[(2) - (3)].p));
@@ -2218,7 +2226,7 @@ yyreduce:
     break;
 
   case 42:
-#line 638 "parse.y"
+#line 646 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    cblock_expr((yyvsp[(1) - (1)].p));
@@ -2227,7 +2235,7 @@ yyreduce:
     break;
 
   case 43:
-#line 645 "parse.y"
+#line 653 "parse.y"
     {
 	  if (!state.mac_prev) {
 	    cblock_expr_incr((yyvsp[(1) - (3)].p), (yyvsp[(3) - (3)].p));
@@ -2236,42 +2244,42 @@ yyreduce:
     break;
 
   case 54:
-#line 658 "parse.y"
+#line 666 "parse.y"
     {
 	  (yyval.p) = mk_list((yyvsp[(1) - (1)].p), NULL);
 	}
     break;
 
   case 55:
-#line 663 "parse.y"
+#line 671 "parse.y"
     {
           (yyval.p) = mk_list((yyvsp[(1) - (2)].p), mk_list(mk_symbol(""), NULL));
 	}
     break;
 
   case 56:
-#line 668 "parse.y"
+#line 676 "parse.y"
     {
 	  (yyval.p) = mk_list((yyvsp[(1) - (3)].p), (yyvsp[(3) - (3)].p));
 	}
     break;
 
   case 57:
-#line 673 "parse.y"
+#line 681 "parse.y"
     {
 	  (yyval.p) = mk_list(mk_symbol(""), (yyvsp[(2) - (2)].p));
 	}
     break;
 
   case 58:
-#line 678 "parse.y"
+#line 686 "parse.y"
     {
 	  (yyval.p) = mk_list(mk_symbol(""), mk_list(mk_symbol(""), NULL));
 	}
     break;
 
   case 60:
-#line 687 "parse.y"
+#line 695 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2280,7 +2288,7 @@ yyreduce:
     break;
 
   case 63:
-#line 700 "parse.y"
+#line 708 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2289,7 +2297,7 @@ yyreduce:
     break;
 
   case 66:
-#line 713 "parse.y"
+#line 721 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2298,7 +2306,7 @@ yyreduce:
     break;
 
   case 69:
-#line 726 "parse.y"
+#line 734 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2307,7 +2315,7 @@ yyreduce:
     break;
 
   case 74:
-#line 739 "parse.y"
+#line 747 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2316,7 +2324,7 @@ yyreduce:
     break;
 
   case 82:
-#line 752 "parse.y"
+#line 760 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2325,7 +2333,7 @@ yyreduce:
     break;
 
   case 86:
-#line 765 "parse.y"
+#line 773 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2334,7 +2342,7 @@ yyreduce:
     break;
 
   case 90:
-#line 778 "parse.y"
+#line 786 "parse.y"
     {
 	  coerce_str1((yyvsp[(1) - (3)].p));
 	  coerce_str1((yyvsp[(3) - (3)].p));
@@ -2343,7 +2351,7 @@ yyreduce:
     break;
 
   case 95:
-#line 791 "parse.y"
+#line 799 "parse.y"
     {
 	  coerce_str1((yyvsp[(2) - (2)].p));
 	  (yyval.p) = mk_1op((yyvsp[(1) - (2)].i), (yyvsp[(2) - (2)].p));
@@ -2351,91 +2359,91 @@ yyreduce:
     break;
 
   case 103:
-#line 801 "parse.y"
+#line 809 "parse.y"
     {
 	  (yyval.p) = (yyvsp[(1) - (1)].p);
         }
     break;
 
   case 104:
-#line 806 "parse.y"
+#line 814 "parse.y"
     {
 	  (yyval.p) = mk_constant((yyvsp[(1) - (1)].i));
 	}
     break;
 
   case 105:
-#line 811 "parse.y"
+#line 819 "parse.y"
     {
 	  (yyval.p) = mk_symbol("$");
 	}
     break;
 
   case 106:
-#line 816 "parse.y"
+#line 824 "parse.y"
     {
 	  (yyval.p) = mk_string((yyvsp[(1) - (1)].s));
 	}
     break;
 
   case 107:
-#line 821 "parse.y"
+#line 829 "parse.y"
     {
 	  (yyval.p) = (yyvsp[(2) - (3)].p);
 	}
     break;
 
   case 108:
-#line 826 "parse.y"
+#line 834 "parse.y"
     {
 	  (yyval.p) = mk_offset((yyvsp[(2) - (3)].p));
 	}
     break;
 
   case 109:
-#line 831 "parse.y"
+#line 839 "parse.y"
     {
 	  (yyval.p) = mk_constant(TBL_NO_CHANGE);
 	}
     break;
 
   case 110:
-#line 836 "parse.y"
+#line 844 "parse.y"
     {
 	  (yyval.p) = mk_constant((yyvsp[(1) - (1)].i));
 	}
     break;
 
   case 111:
-#line 841 "parse.y"
+#line 849 "parse.y"
     {
 	  (yyval.p) = mk_constant((yyvsp[(1) - (1)].i));
 	}
     break;
 
   case 112:
-#line 846 "parse.y"
+#line 854 "parse.y"
     {
 	  (yyval.p) = mk_constant((yyvsp[(1) - (1)].i));
 	}
     break;
 
   case 113:
-#line 853 "parse.y"
+#line 861 "parse.y"
     { 
           (yyval.p) = mk_symbol((yyvsp[(1) - (1)].s));
         }
     break;
 
   case 114:
-#line 858 "parse.y"
+#line 866 "parse.y"
     {
           (yyval.p) = mk_2op(CONCAT, mk_symbol((yyvsp[(1) - (3)].s)), mk_1op(VAR, (yyvsp[(2) - (3)].p)));
         }
     break;
 
   case 115:
-#line 863 "parse.y"
+#line 871 "parse.y"
     {
           (yyval.p) = mk_2op(CONCAT, mk_symbol((yyvsp[(1) - (3)].s)), 
                         mk_2op(CONCAT, mk_1op(VAR, (yyvsp[(2) - (3)].p)), mk_symbol((yyvsp[(3) - (3)].s))));
@@ -2443,48 +2451,60 @@ yyreduce:
     break;
 
   case 116:
-#line 871 "parse.y"
+#line 879 "parse.y"
     { 
           (yyval.s) = (yyvsp[(1) - (1)].s);
+
+	  /*
+	   * statements return their org - but, with 16bit cores, org is a word
+	   * address. for us to know whether a label points at a non word aligned
+	   * address, we must get status from the directive.c module through a
+	   * back channel. however, we must make sure to store this status before
+	   * any statement on the current line is processed, so we must save it here
+	   * before the statement rules run.
+	   */
+	  _16packed_offset_labels = _16packed_byte_acc;
         }
     break;
 
   case 117:
-#line 876 "parse.y"
+#line 894 "parse.y"
     {
           if (asm_enabled() && !state.mac_prev) {
 	    (yyval.s) = evaluate_concatenation(mk_2op(CONCAT,  mk_symbol((yyvsp[(1) - (3)].s)), 
                            mk_1op(VAR, (yyvsp[(2) - (3)].p))));
+	  _16packed_offset_labels = _16packed_byte_acc;
 	  }
         }
     break;
 
   case 118:
-#line 884 "parse.y"
+#line 903 "parse.y"
     {
           if (asm_enabled() && !state.mac_prev) {
             (yyval.s) = evaluate_concatenation(mk_2op(CONCAT,  mk_symbol((yyvsp[(1) - (3)].s)), 
                       mk_2op(CONCAT, mk_1op(VAR, (yyvsp[(2) - (3)].p)), mk_symbol((yyvsp[(3) - (3)].s)))));
+	  _16packed_offset_labels = _16packed_byte_acc;
 	  }
         }
     break;
 
   case 119:
-#line 894 "parse.y"
+#line 914 "parse.y"
     {
 	  (yyval.p) = mk_list((yyvsp[(1) - (1)].p), NULL);
 	}
     break;
 
   case 120:
-#line 899 "parse.y"
+#line 919 "parse.y"
     {
 	  (yyval.p) = mk_list((yyvsp[(1) - (3)].p), (yyvsp[(3) - (3)].p));
 	}
     break;
 
   case 121:
-#line 906 "parse.y"
+#line 926 "parse.y"
     {
 	  if ((strcasecmp((yyvsp[(1) - (1)].s), "p") == 0) || (strcasecmp((yyvsp[(1) - (1)].s), "pe") == 0)) { 
             force_ident = 1;
@@ -2493,7 +2513,7 @@ yyreduce:
     break;
 
   case 122:
-#line 912 "parse.y"
+#line 932 "parse.y"
     {
 	  (yyval.p) = mk_2op((yyvsp[(3) - (4)].i), mk_symbol((yyvsp[(1) - (4)].s)), (yyvsp[(4) - (4)].p));
 	  force_ident = 0;
@@ -2501,7 +2521,7 @@ yyreduce:
     break;
 
   case 123:
-#line 918 "parse.y"
+#line 938 "parse.y"
     {
 	  (yyval.p) = (yyvsp[(1) - (1)].p);
 	}
@@ -2509,7 +2529,7 @@ yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 2513 "parse.c"
+#line 2533 "parse.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2723,6 +2743,6 @@ yyreturn:
 }
 
 
-#line 923 "parse.y"
+#line 943 "parse.y"
 
 
