@@ -1,5 +1,5 @@
 /* Displays contents of ".COD" files
-   Copyright (C) 2001, 2002, 2003, 2004
+   Copyright (C) 2001, 2002, 2003, 2004, 2005
    Scott Dattalo
 
 This file is part of gputils.
@@ -36,7 +36,7 @@ extern char *source_file_names[MAX_SOURCE_FILES];
 extern FILE *source_files[MAX_SOURCE_FILES];
 extern DirBlockInfo main_dir;
 
-extern int addrsize;
+extern int byte_addr;
 
 extern char directory_block_data[COD_BLOCK_SIZE];
 extern char * SymbolType4[154];
