@@ -244,12 +244,14 @@ enum pic_processor {
   pic18f1330,
   pic18f2220,
   pic18f2320,
+  pic18f2321,
   pic18f2331,
   pic18f2410,
   pic18f242,
   pic18f2420,
   pic18f2431,
   pic18f2439,
+  pic18f2450,
   pic18f2455,
   pic18f248,
   pic18f2480,
@@ -257,6 +259,7 @@ enum pic_processor {
   pic18f2510,
   pic18f2515,
   pic18f252,
+  pic18f2523,
   pic18f2525,
   pic18f2539,
   pic18f2550,
@@ -273,12 +276,14 @@ enum pic_processor {
   pic18f2685,
   pic18f4220,
   pic18f4320,
+  pic18f4321,
   pic18f4331,
   pic18f4410,
   pic18f442,
   pic18f4420,
   pic18f4431,
   pic18f4439,
+  pic18f4450,
   pic18f4455,
   pic18f448,
   pic18f4480,
@@ -287,6 +292,7 @@ enum pic_processor {
   pic18f4515,
   pic18f452,
   pic18f4520,
+  pic18f45k20,
   pic18f4525,
   pic18f4539,
   pic18f4550,
@@ -307,18 +313,22 @@ enum pic_processor {
   pic18f64j15,
   pic18f6520,
   pic18f6525,
-  pic18f6585,
   pic18f65j10,
   pic18f65j15,
+  pic18f65j50,
+  pic18f6585,
   pic18f6620,
   pic18f6621,
   pic18f6627,
   pic18f6680,
   pic18f66j10,
   pic18f66j15,
+  pic18f66j60,
+  pic18f66j65,
   pic18f6720,
   pic18f6722,
   pic18f67j10,
+  pic18f67j60,
   pic18f8310,
   pic18f8390,
   pic18f8410,
@@ -335,9 +345,15 @@ enum pic_processor {
   pic18f8680,
   pic18f86j10,
   pic18f86j15,
+  pic18f86j60,
+  pic18f86j65,
   pic18f8720,
   pic18f8722,
   pic18f87j10,
+  pic18f87j60,
+  pic18f96j60,
+  pic18f96j65,
+  pic18f97j60,
   rf509af,	
   rf509ag,	
   rf675f,
@@ -351,6 +367,8 @@ enum pic_processor {
 };
 
 #define MAX_NAMES 3 /* Maximum number of names a processor can have */
+#define MAX_BADROM 1*2 /* Maximum number of BADROM ranges a processor can be */
+                       /* initialized with */
 
 struct px {
   enum pic_processor tag;
@@ -361,6 +379,8 @@ struct px {
   int num_pages;
   int num_banks;
   long maxrom;
+  long badrom[MAX_BADROM];
+  long config_addrs[2];
   char *script;
 };
 
